@@ -6,3 +6,5 @@ class AuthType(Enum):
 
     APIKey = "APIKey"
     """ bearer authentication - using token representing a user in your tenant """
+    OAuth2 = "OAuth2"
+    """ OAuth2 authentication flows """
