@@ -7,13 +7,14 @@ class RetryResponseInterceptor:
 
     def intercept(self, res):
         request_data = res["request_data"]
+        # request_data = req
         # retry on request_aborted for get methods, except GetHyperCube and GetListObject
         # - exclude GetHyperCube* and GetListObject*.
         # - if they are aborted a new layout need to be fetched first to know if retrying the same parameter is relevant
         is_get_method = bool(
             re.match(
                 r"(?!^GetHyperCube|^GetListObject)(^Get)",
-                request_data["method"],
+                request_data.method,
                 flags=re.IGNORECASE,
             )
         )
