@@ -1,15 +1,30 @@
-from .auth import Auth
-from .config import Config
-from .generated import (
+from .apis import (
+    Api_Keys,
     Apps,
+    Audits,
+    Automations,
+    Collections,
+    Csp_Origins,
+    Data_Files,
     Extensions,
+    Groups,
+    Identity_Providers,
     Items,
-    Qix_Datafiles,
+    Licenses,
+    Oauth_Tokens,
+    Questions,
+    Quotas,
+    Reload_Tasks,
     Reloads,
+    Roles,
     Spaces,
     Themes,
     Users,
+    Web_Integrations,
+    Webhooks,
 )
+from .auth import Auth
+from .config import Config
 from .rest import RestClientInstance
 from .rpc import RpcClientInstance
 
@@ -104,7 +119,22 @@ class Qlik:
     extensions: Extensions.Extensions
     reloads: Reloads.Reloads
     themes: Themes.Themes
-    datafiles: Qix_Datafiles.QixDatafiles
+    datafiles: Data_Files.DataFiles
+    roles: Roles.Roles
+    groups: Groups.Groups
+    licenses: Licenses.Licenses
+    identityproviders: Identity_Providers.IdentityProviders
+    csporigins: Csp_Origins.CspOrigins
+    webintegrations: Web_Integrations.WebIntegrations
+    apikeys: Api_Keys.ApiKeys
+    audits: Audits.Audits
+    quotas: Quotas.Quotas
+    webhooks: Webhooks.Webhooks
+    automations: Automations.Automations
+    collections: Collections.Collections
+    reload_tasks: Reload_Tasks.ReloadTasks
+    oauth_tokens: Oauth_Tokens.OauthTokens
+    questions: Questions.Questions
 
     def __init__(self, config: Config) -> None:
         self.config = config
@@ -118,4 +148,19 @@ class Qlik:
         self.users = Users.Users(config=config)
         self.reloads = Reloads.Reloads(config=config)
         self.themes = Themes.Themes(config=config)
-        self.datafiles = Qix_Datafiles.QixDatafiles(config=config)
+        self.datafiles = Data_Files.DataFiles(config=config)
+        self.roles = Roles.Roles(config=config)
+        self.groups = Groups.Groups(config=config)
+        self.licenses = Licenses.Licenses(config=config)
+        self.identityproviders = Identity_Providers.IdentityProviders(config=config)
+        self.csporigins = Csp_Origins.CspOrigins(config=config)
+        self.webintegrations = Web_Integrations.WebIntegrations(config=config)
+        self.apikeys = Api_Keys.ApiKeys(config=config)
+        self.audits = Audits.Audits(config=config)
+        self.quotas = Quotas.Quotas(config=config)
+        self.webhooks = Webhooks.Webhooks(config=config)
+        self.automations = Automations.Automations(config=config)
+        self.collections = Collections.Collections(config=config)
+        self.reload_tasks = Reload_Tasks.ReloadTasks(config=config)
+        self.oauth_tokens = Oauth_Tokens.OauthTokens(config=config)
+        self.questions = Questions.Questions(config=config)
