@@ -1,17 +1,45 @@
 import json
-from dataclasses import asdict, dataclass
+import struct
+from dataclasses import dataclass
 from functools import reduce
 from threading import Condition, Lock, Thread
-from typing import Callable, Dict, List
+from typing import Callable, Dict, List, Literal
 from urllib.parse import urlparse
 
 import websocket
 
 from ._version import __version__
 from .config import Config
+from .errors import EngineClosedException
 from .interceptors.types import InterceptorHandler, Interceptors
 
 
+def _get_json_data(x: any):
+    """
+    Get all the properties of a class as plain data.
+    Similar to the asdict function but including dynamic properties.
+    Excludes _session property
+    """
+    dict_data = None
+    try:
+        # get the __dict__ of the object
+        # includes named and dynamic properties
+        dict_data = x.__dict__
+    except:
+        if type(x) is dict:
+            dict_data = x
+    if dict_data is not None:
+        dct = {}
+        for k, v in dict_data.items():
+            # exclude the private _session property
+            if k != "_session":
+                dct[k] = _get_json_data(v)
+        return dct
+    elif type(x) is list:
+        return [_get_json_data(e) for e in x]
+    return x
+
+
 @dataclass
 class RequestObject:
     # Identifier established by the initiator of the request
@@ -51,12 +79,11 @@ class FailedToConnect(Exception):
 
 
 class RpcSession:
-
     _interceptors: Interceptors[ResponseInterceptor, RequestInterceptor]
 
     _ws_url: str
     _headers: List[str]
-
+    _recv_error: Exception
     _socket = None
     _watch_recv_thread = None
 
@@ -71,6 +98,28 @@ class RpcSession:
         self._ws_url = ws_url
         self._interceptors = interceptors
         self.lock = Lock()
+        self._listeners = {}
+        self._id_listeners = {}
+        self._recv_error = None
+
+    def _emit(self, event, *args):
+        if event in self._listeners:
+            self._listeners[event](*args)
+        if event == "closed":
+            # clear all listeners if session is closed
+            self._listeners = {}
+            self._id_listeners = {}
+
+    def _emit_handle(self, handle, event, *args):
+        if handle in self._id_listeners and event in self._id_listeners[handle]:
+            self._id_listeners[handle][event](*args)
+        if event == "closed":
+            # clear all listeners if handle is closed
+            self._clear_listeners(handle)
+
+    def _emit_handles(self, handles, event, *args):
+        for id in handles:
+            self._emit_handle(id, event, *args)
 
     def _watch_recv(self):
         """
@@ -82,18 +131,46 @@ class RpcSession:
             if not self.is_connected():
                 return
             try:
-                res = self._socket.recv()
+                # Using code from the websocket-library documentation for receiving connection close status code
+                # https://websocket-client.readthedocs.io/en/latest/examples.html#receiving-connection-close-status-codes
+                resp_opcode, msg = self._socket.recv_data()
+                if resp_opcode == 8 and len(msg) >= 2:
+                    close_code = str(struct.unpack("!H", msg[0:2])[0])
+                    close_msg = str(msg[2:])
+                    self._socket = None
+                    msg = False
+                    # Assume that only close code >= 4000 is error
+                    if int(close_code) >= 4000:
+                        # Setting socket to None signals not connected which will close this thread
+                        # The error will be thrown in the close function not in this thread because then
+                        # it will not reach the caller
+                        self._recv_error = EngineClosedException(
+                            {"close_code": close_code, "close_msg": close_msg}
+                        )
             except Exception as err:
                 self._socket = None
-                res = False
+                msg = False
                 self._recv_error = err
             with self._received_added:
-                if res:
-                    res = json.loads(res)
+                if msg:
+                    msg = json.loads(msg)
+                    self._emit("traffic:*", "received", msg)
+                    self._emit("traffic:received", msg)
                     # add response to _received and notify waiting
-                    if "id" in res:
-                        self._received[res["id"]] = res
+                    if "id" in msg:
+                        self._received[msg["id"]] = msg
+                        self._emit("message", msg)
+                        self._emit_handle(msg["id"], "traffic:*", "received", msg)
+                        self._emit_handle(msg["id"], "traffic:received", msg)
                         self._received_added.notify_all()
+                    else:
+                        self._emit(
+                            "notification" if "params" in msg else "message", msg
+                        )
+                    if "change" in msg:
+                        self._emit_handles(msg["change"], "changed")
+                    if "close" in msg:
+                        self._emit_handles(msg["close"], "closed")
                 else:
                     # notify waiting receivers so that
                     # the not connected error can be raised
@@ -101,6 +178,50 @@ class RpcSession:
                     # wait_response will never finish
                     self._received_added.notify_all()
 
+    def on(
+        self,
+        event_name: Literal[
+            "traffic:*",
+            "traffic:received",
+            "traffic:sent",
+            "closed",
+            "opened",
+            "notification",
+            "message",
+        ],
+        listener: Callable[[any], None],
+    ):
+        """
+        the on function handles the various states and communication events
+
+        Parameters
+        ----------
+        event_name: str
+            can be one of
+            'traffic', 'traffic:received','traffic:sent',
+            'closed', 'opened',
+            'notification', 'message'
+
+        Examples
+        --------
+        >>> from qlik_sdk import Auth, AuthType, Config
+        ...
+        ... rpc_session.on("closed", closed_listener)
+        ... rpc_session.on("opened", opened_listener)
+        ... with rpc_session.open() as rpc_client:
+        ...     # ...
+        """
+        self._listeners[event_name] = listener
+
+    def on_handle(self, handle, event_name, listener):
+        if handle not in self._id_listeners:
+            self._id_listeners[handle] = {}
+        self._id_listeners[handle][event_name] = listener
+
+    def _clear_listeners(self, handle):
+        if handle in self._id_listeners:
+            self._id_listeners[handle] = {}
+
     def open(self):
         """
         connect establishes a connection to provided url
@@ -124,6 +245,7 @@ class RpcSession:
 
         self._watch_recv_thread = Thread(target=self._watch_recv)
         self._watch_recv_thread.start()
+        self._emit("opened")
         return self
 
     def is_connected(self):
@@ -139,9 +261,14 @@ class RpcSession:
 
         if self.is_connected():
             self._socket.send_close()
+            self._emit("closed")
         if self._watch_recv_thread is not None and self._watch_recv_thread.is_alive():
             self._watch_recv_thread.join()
 
+        # Raise closed connection error here
+        if self._recv_error:
+            raise self._recv_error
+
     def __enter__(self):
         """
         __enter__ is called when client is used in a 'with' statement.
@@ -193,8 +320,12 @@ class RpcSession:
 
         # send and wait respons
         data = reduce(lambda d, f: f(d), self._interceptors["request"].handlers, data)
-        json_data = json.dumps(asdict(data))
+        json_data = json.dumps(_get_json_data(data))
         self._socket.send(json_data)
+        self._emit("traffic:*", "sent", data)
+        self._emit("traffic:sent", data)
+        self._emit_handle(handle, "traffic:*", "sent", data)
+        self._emit_handle(handle, "traffic:sent", data)
         res = self._wait_response(id_)
         res["request_data"] = data
         res = reduce(lambda r, f: f(r), self._interceptors["response"].handlers, res)
